package cn.com.qmth.examcloud.core.oe.student.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.core.oe.student.bean.client.CourseInfo;
import cn.com.qmth.examcloud.core.oe.student.service.*;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.FileAnswerAcknowledgeStatus;
import cn.com.qmth.examcloud.support.examing.ExamFileAnswer;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.filestorage.FileStorageHelper;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.FileStorageType;
import cn.com.qmth.examcloud.web.filestorage.YunHttpRequest;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;
import com.google.common.collect.Maps;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.math.RandomUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Api(tags = "在线考试控制")
@RestController
@RequestMapping("${app.api.oe.student}/examControl")
public class ExamControlController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ExamControlController.class);

    private static final String SEPARATOR = "/";

    private static final String UNDERLINE = "_";

    // 小程序答案上传目录
    private static final String OE_ANSWER_FILE_PATH = "oe-answer-file";

    @Autowired
    private ExamControlService examControlService;

    @Autowired
    private ExamFileAnswerService examFileAnswerService;

    @Autowired
    private UpyunService upyunService;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private TempConfigService tempConfigService;

    /**
     * 开始考试
     */
    @Deprecated
    @ApiOperation(value = "开始考试（待废弃）")
    @GetMapping("/startExam")
    public StartExamInfo startExam(@RequestParam Long examStudentId, HttpServletRequest request) {
        User user = getAccessUser();
        if (tempConfigService.checkDisabledForOldApi(user.getRootOrgId())) {
            // 判断学校是否被禁用旧版本API
            throw new StatusException("403X01", "禁止访问！");
        }

        return examControlService.startExam(examStudentId, user.getUserId(), getIp(request), null);
    }

    /**
     * 开始考试（新）
     */
    @ApiOperation(value = "开始考试（不支持“ONLINE”考试类型）")
    @PostMapping("/unonline/startExam")
    public StartExamInfo startExamForOther(@RequestParam Long examStudentId, HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.startExam(examStudentId, user.getUserId(), getIp(request), false);
    }

    /**
     * 开始考试（新）
     * 注：需要加、解密
     */
    @ApiOperation(value = "开始考试（仅支持“ONLINE”考试类型）")
    @PostMapping(value = "/online/startExam", consumes = MediaType.TEXT_PLAIN_VALUE, produces = MediaType.TEXT_PLAIN_VALUE)
    public String startExamWithCrypto(@RequestBody String encryptParams, HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.startExamWithCrypto(encryptParams, user, getIp(request),
                request.getHeader(CryptoConstant.TIMESTAMP));
    }

    /**
     * 开始答题
     */
    @ApiOperation(value = "开始答题")
    @PostMapping("/startAnswer")
    public StartAnswerInfo startAnswer(@RequestParam @ApiParam(value = "考试记录id") Long examRecordDataId) {
        User user = getAccessUser();
        return examControlService.startAnswer(examRecordDataId, user.getUserId());
    }

    /**
     * 考生作答（新）
     * 注：需要加、解密
     */
    @ApiOperation(value = "考试过程中-考生试题作答（新）")
    @PostMapping("/submitQuestionAnswer")
    public void submitQuestionAnswerWithCrypto(@RequestBody ExamStudentQuestionAnswerInfo data, HttpServletRequest request) {
        User user = getAccessUser();
        String referer = request.getHeader("REFERER");
        String agent = request.getHeader("USER-AGENT");
        String timestampStr = request.getHeader(CryptoConstant.TIMESTAMP);
        examRecordQuestionsService.submitQuestionAnswerWithCrypto(data, user, referer, agent, timestampStr, getIp(request));
    }

    /**
     * 断点续考:检查正在进行中的考试
     */
    @ApiOperation(value = "断点续考:检查正在进行中的考试")
    @GetMapping("/checkExamInProgress")
    public ExamProcessResultInfo checkExamInProgress(HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.checkExamInProgress2(user.getUserId(), getIp(request));
    }

    /**
     * 考试心跳
     *
     * @return 剩余时间
     */
    @ApiOperation(value = "考试心跳")
    @GetMapping("/examHeartbeat")
    public Long examHeartbeat(HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.examHeartbeat(user, getIp(request));
    }

    /**
     * 结束考试：交卷
     */
    @Deprecated
    @ApiOperation(value = "结束考试：交卷（待废弃）")
    @GetMapping("/endExam")
    public void endExam(@RequestParam(required = false) Boolean force, HttpServletRequest request) {
        User user = getAccessUser();
        examControlService.manualEndExam(user.getUserId(), getIp(request), force);
    }

    /**
     * 结束考试：交卷（新）
     * 注：需要加、解密
     */
    @ApiOperation(value = "结束考试：交卷（新）")
    @PostMapping(value = "/endExam", consumes = MediaType.TEXT_PLAIN_VALUE)
    public void endExamWithCrypto(@RequestBody String encryptParams, HttpServletRequest request) {
        User user = getAccessUser();
        examControlService.manualEndExamWithCrypto(encryptParams, user, getIp(request),
                request.getHeader(CryptoConstant.TIMESTAMP));
    }

    /**
     * 获取考试记录信息
     */
    @ApiOperation(value = "获取考试记录信息")
    @GetMapping("/getEndExamInfo")
    public EndExamInfo getEndExamInfo(@RequestParam Long examRecordDataId) {
        return examControlService.getEndExamInfo(examRecordDataId);
    }

    /**
     * 获取又拍云文件上传签名（微信小程序调用）
     */
    @ApiOperation(value = "获取又拍云文件上传签名（微信小程序调用）")
    @PostMapping("/upyunSignature")
    public UpyunSignatureInfo getUpyunSignature(@ModelAttribute @Valid GetYunSignatureReq req) {
        return examControlService.getUpyunSignature(req);
    }

    /**
     * 获取云存储上传签名（微信小程序调用）
     */
    @ApiOperation(value = "获取文件上传签名（微信小程序调用）")
    @PostMapping("/yunSignature")
    public GetYunSignDomain getYunSignature(@ModelAttribute @Valid GetYunSignatureReq req) {
        FileStorageType fileStorageType = FileStorageUtil.getFileStorageType();

        // 是否为环境监测场景
        boolean isTestDev = Constants.OE_ENV_CHECK_EXAM_RECORD_DATA_ID.equals(req.getExamRecordDataId());

        if (FileStorageType.UPYUN.equals(fileStorageType)) {
            GetUpyunSignDomain result = new GetUpyunSignDomain();
            Map<String, String> params = Maps.newHashMap();
            UpyunSignatureInfo info = examControlService.getUpyunSignature(req);
            String signIdentifier = String.valueOf(System.currentTimeMillis());
            params.put("authorization", info.getSignature());
            params.put("policy", info.getPolicy());
            result.setAccessUrl(FileStorageHelper.getUrl(info.getUpyunFileDomain(), info.getFilePath()));
            result.setFormUrl(info.getUploadUrl());
            result.setFormParams(params);
            result.setSignIdentifier(signIdentifier);
            return result;
        }

        if (FileStorageType.ALIYUN.equals(fileStorageType)) {
            String fileSuffix = req.getFileSuffix();
            if (StringUtils.isNullOrEmpty(fileSuffix)) {
                throw new StatusException("5002", "文件后缀名不允许为空");
            }
            fileSuffix = !fileSuffix.contains(".") ? "." + fileSuffix : fileSuffix;

            Long examStudentId;
            if (isTestDev) {
                examStudentId = Constants.OE_ENV_CHECK_EXAM_STUDENT_ID;
            } else {
                ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(req.getExamRecordDataId());
                if (examRecordData == null) {
                    log.warn("ExamRecordDataCache not exist! examRecordDataId = {}", req.getExamRecordDataId());
                    throw new StatusException("5002", "请求参数值有误！");
                }
                examStudentId = examRecordData.getExamStudentId();
            }

            StringBuffer filePath = new StringBuffer();
            filePath.append(OE_ANSWER_FILE_PATH).append(SEPARATOR)
                    .append(examStudentId).append(SEPARATOR).append(req.getExamRecordDataId())
                    .append(SEPARATOR).append(req.getOrder()).append(SEPARATOR)
                    .append(examStudentId).append(UNDERLINE).append(req.getExamRecordDataId())
                    .append(UNDERLINE).append(req.getOrder()).append(UNDERLINE).append(System.currentTimeMillis())
                    .append(RandomUtils.nextInt(8999) + 1000);

            if (!StringUtils.isNullOrEmpty(req.getExt())) {
                filePath.append(UNDERLINE).append(req.getExt());
            }
            filePath.append(fileSuffix);

            GetAliyunSignDomain result = new GetAliyunSignDomain();
            String signIdentifier = String.valueOf(System.currentTimeMillis());
            FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
            env.setRelativePath(filePath.toString());

            YunHttpRequest aliYunHttpRequest = FileStorageUtil.getSignature(FileStorageType.ALIYUN, Constants.MINI_PROGRAM_ANWSER_SITEID, env, req.getFileMd5());
            result.setAccessUrl(aliYunHttpRequest.getAccessUrl());
            result.setFormUrl(aliYunHttpRequest.getFormUrl());
            result.setFormParams(aliYunHttpRequest.getFormParams());
            result.setSignIdentifier(signIdentifier);
            return result;
        }

        throw new StatusException("5002", "未配置正确云存储类型");
    }

    /**
     * 校验二维码（微信小程序调用）
     */
    @Naked
    @ApiOperation(value = "校验二维码（微信小程序调用）")
    @PostMapping("/checkQrCode")
    public CheckQrCodeInfo checkQrCode(@RequestParam(required = true) String qrCode) {
        return examControlService.checkQrCode(qrCode);
    }

    /**
     * 保存上传的文件（微信小程序调用）
     */
    @ApiOperation(value = "保存上传的文件（微信小程序调用）")
    @PostMapping("/saveUploadedFile")
    public String saveUploadedFile(@ModelAttribute @Valid SaveUploadedFileReq req) {
        User user = getAccessUser();
        ExamFileAnswer examFileAnswer = new ExamFileAnswer();
        examFileAnswer.setExamRecordDataId(req.getExamRecordDataId());
        examFileAnswer.setExamStudentId(req.getExamStudentId());
        examFileAnswer.setQuestionOrder(req.getOrder());
        examFileAnswer.setFilePath(req.getFilePath());
        examFileAnswer.setStatus(FileAnswerAcknowledgeStatus.UNCONFIRMED);
        examFileAnswer.setTransferFileType(req.getTransferFileType());
        String fileAnswerId = RedisKeyHelper.getBuilder().studentFileAnswerKey(req.getExamRecordDataId(), req.getOrder());
        examFileAnswerService.saveFileAnswer(fileAnswerId, examFileAnswer);

        try {
            String fileUrl = "";
            if (req.getFilePath().indexOf(",") > -1) {
                for (String url : req.getFilePath().split(",")) {
                    fileUrl += FileStorageUtil.realPath(url) + ",";
                }
                fileUrl = fileUrl.substring(0, fileUrl.length() - 1);
            } else {
                fileUrl = FileStorageUtil.realPath(req.getFilePath());
            }
            examControlService.sendFileAnswerToWebSocket(req.getExamRecordDataId(), req.getOrder(),
                    fileUrl, req.getTransferFileType(), user.getUserId(), user.getRootOrgId());
        } catch (Exception e) {
            examFileAnswerService.deleteFileAnswer(fileAnswerId);
            throw new StatusException("100009", "消息通知失败", e);
        }
        return fileAnswerId;
    }


    /**
     * 查询客户端对上传的文件的响应状态（微信小程序调用）
     */
    @ApiOperation(value = "查询客户端对上传的文件的响应状态（微信小程序调用）")
    @PostMapping("/getUploadedFileAcknowledgeStatus")
    public String getUploadedFileAcknowledgeStatus(@ModelAttribute @Valid GetUploadedFileAcknowledgeStatusReq req) {
        ExamFileAnswer fileAnswer = examFileAnswerService.getFileAnswer(req.getAcknowledgeId());
        if (null != fileAnswer) {
            return fileAnswer.getStatus().toString();
        }
        return FileAnswerAcknowledgeStatus.UNCONFIRMED.toString();
    }

    /**
     * 修改上传音频结果推送状态
     */
    @ApiOperation(value = "修改上传音频结果推送状态")
    @PostMapping("/saveUploadedFileAcknowledgeStatus")
    public void saveUploadedFileAcknowledgeStatus(@RequestBody @Valid SaveUploadedFileAcknowledgeStatusReq req) {
        String acknowledgeId = RedisKeyHelper.getBuilder().studentFileAnswerKey(req.getExamRecordDataId(), req.getOrder());
        ExamFileAnswer fileAnswer = examFileAnswerService.getFileAnswer(acknowledgeId);
        if (null == fileAnswer) {
            throw new StatusException("100010", "无效的数据");
        }

        fileAnswer.setStatus(FileAnswerAcknowledgeStatus.valueOf(req.getAcknowledgeStatus()));
        examFileAnswerService.saveFileAnswer(acknowledgeId, fileAnswer);
    }

    //原接口，不作改动
    @ApiOperation(value = "获取抓拍照片的又拍云签名")
    @GetMapping("/getCapturePhotoUpYunSign")
    public GetUpyunSignDomain getCapturePhotoUpYunSign(GetYunSignDomainQuery query) {
        return getUpYunSign(query);
    }

    //又拍云签名
    private GetUpyunSignDomain getUpYunSign(GetYunSignDomainQuery query) {
        String fileSuffix = query.getFileSuffix();
        if (StringUtils.isNullOrEmpty(fileSuffix)) {
            throw new StatusException("200001", "文件后缀名不允许为空");
        }
        fileSuffix = fileSuffix.indexOf(".") == -1 ? "." + fileSuffix : fileSuffix;

        GetUpyunSignDomain result = new GetUpyunSignDomain();
        User accessUser = this.getAccessUser();
        String signIdentifier = String.valueOf(System.currentTimeMillis());
        String upyunSignRedisKey = CacheConstants.CACHE_OE_UPYUN_IMAGE_SIGN
                + accessUser.getUserId() + "_" + signIdentifier;

        UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
        env.setRootOrgId(accessUser.getRootOrgId().toString());
        env.setUserId(accessUser.getUserId().toString());
        env.setFileSuffix(fileSuffix);
        YunHttpRequest upYunHttpRequest = upyunService.buildUpYunHttpRequest(Constants.CAPTURE_PHOTO_UPYUN_SITEID, env, query.getFileMd5());
        redisClient.set(upyunSignRedisKey, upYunHttpRequest, 60);
        result.setAccessUrl(upYunHttpRequest.getAccessUrl());
        result.setFormUrl(upYunHttpRequest.getFormUrl());
        result.setFormParams(upYunHttpRequest.getFormParams());
        result.setSignIdentifier(signIdentifier);
        return result;
    }

    @ApiOperation(value = "获取抓拍照片的云存储签名")
    @GetMapping("/getCapturePhotoYunSign")
    public GetYunSignDomain getCapturePhotoYunSign(GetYunSignDomainQuery query) {
        if (FileStorageType.UPYUN.equals(FileStorageUtil.getFileStorageType())) {
            return getUpYunSign(query);
        }
        if (FileStorageType.ALIYUN.equals(FileStorageUtil.getFileStorageType())) {
            return getAliYunSign(query);
        }
        throw new StatusException("3002", "未配置正确云存储类型");
    }

    //阿里云签名
    private GetAliyunSignDomain getAliYunSign(GetYunSignDomainQuery query) {
        String fileSuffix = query.getFileSuffix();
        if (StringUtils.isNullOrEmpty(fileSuffix)) {
            throw new StatusException("4001", "文件后缀名不允许为空");
        }
        fileSuffix = fileSuffix.indexOf(".") == -1 ? "." + fileSuffix : fileSuffix;

        GetAliyunSignDomain result = new GetAliyunSignDomain();
        User accessUser = this.getAccessUser();
        String signIdentifier = String.valueOf(System.currentTimeMillis());
        String aliyunSignRedisKey = CacheConstants.CACHE_OE_UPYUN_IMAGE_SIGN
                + accessUser.getUserId() + "_" + signIdentifier;
        FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
        env.setRootOrgId(accessUser.getRootOrgId().toString());
        env.setUserId(accessUser.getUserId().toString());
        env.setFileSuffix(fileSuffix);
        YunHttpRequest aliYunHttpRequest = FileStorageUtil.getSignature(FileStorageType.ALIYUN, Constants.CAPTURE_PHOTO_UPYUN_SITEID, env, query.getFileMd5());
        redisClient.set(aliyunSignRedisKey, aliYunHttpRequest, 60);
        result.setAccessUrl(aliYunHttpRequest.getAccessUrl());
        result.setFormUrl(aliYunHttpRequest.getFormUrl());
        result.setFormParams(aliYunHttpRequest.getFormParams());
        result.setSignIdentifier(signIdentifier);
        return result;
    }

    @ApiOperation(value = "批量获取抓拍照片的又拍云签名")
    @PostMapping("/batchGetCapturePhotoUpYunSign")
    public BatchGetUpyunSignDomain batchGetCapturePhotoUpYunSign(@RequestBody BatchGetUpyunSignDomainQuery batchQuery) {
        if (batchQuery.getQueryList() == null || batchQuery.getQueryList().isEmpty()) {
            throw new StatusException("300001", "查询条件不允许为空");
        }

        List<GetUpyunSignDomain> signDomainList = new ArrayList<>();
        for (GetUpyunSignDomainQuery query : batchQuery.getQueryList()) {
            String fileSuffix = query.getFileSuffix();
            if (StringUtils.isNullOrEmpty(fileSuffix)) {
                throw new StatusException("", "文件后缀名不允许为空");
            }
            fileSuffix = fileSuffix.indexOf(".") == -1 ? "." + fileSuffix : fileSuffix;

            GetUpyunSignDomain upyunSignDomain = new GetUpyunSignDomain();
            User accessUser = this.getAccessUser();
            String signIdentifier = String.valueOf(System.currentTimeMillis());
            String upyunSignRedisKey = CacheConstants.CACHE_OE_UPYUN_IMAGE_SIGN
                    + accessUser.getUserId() + "_" + signIdentifier;

            UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
            env.setRootOrgId(accessUser.getRootOrgId().toString());
            env.setUserId(accessUser.getUserId().toString());
            env.setFileSuffix(fileSuffix);
            YunHttpRequest upYunHttpRequest = upyunService.buildUpYunHttpRequest(Constants.CAPTURE_PHOTO_UPYUN_SITEID, env, query.getFileMd5());
            redisClient.set(upyunSignRedisKey, upYunHttpRequest, 60);
            upyunSignDomain.setAccessUrl(upYunHttpRequest.getAccessUrl());
            upyunSignDomain.setFormUrl(upYunHttpRequest.getFormUrl());
            upyunSignDomain.setFormParams(upYunHttpRequest.getFormParams());
            upyunSignDomain.setSignIdentifier(signIdentifier);

            signDomainList.add(upyunSignDomain);
            Util.sleep(TimeUnit.MILLISECONDS, 1);
        }
        BatchGetUpyunSignDomain result = new BatchGetUpyunSignDomain();
        result.setList(signDomainList);
        return result;
    }

    @ApiOperation(value = "获取二维码")
    @PostMapping("/getQrCode")
    public String getQrCode(@RequestBody GetQrCodeReq req) {
        return examControlService.getQrCode(req, getAccessUser());
    }

    /**
     * 记录切换屏幕
     */
    @ApiOperation(value = "记录切换屏幕次数")
    @PostMapping("/switchScreen")
    public SwitchScreenCountInfo switchScreen(@RequestParam @ApiParam(value = "考试记录id") Long examRecordDataId) {
        User user = getAccessUser();
        String sequenceLockKey = CacheConstants.LOCK_EXAM_CONTROL + user.getUserId();
        // 开始考试上锁,分布式锁,系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");

        return examControlService.switchScreen(examRecordDataId);
    }

    @ApiOperation(value = "获取课程信息")
    @PostMapping("/getCourseInfo/{examRecordDataId}")
    public CourseInfo courseInfo(@PathVariable Long examRecordDataId) {
        return examRecordDataService.getCourseInfo(examRecordDataId);
    }

    @Deprecated
    @ApiOperation(value = "获取课程名称（待废弃）")
    @GetMapping("/courseName/{examRecordDataId}")
    public String courseName(@PathVariable Long examRecordDataId) {
        CourseInfo info = examRecordDataService.getCourseInfo(examRecordDataId);
        if (info != null) {
            return info.getCourseName();
        }
        return null;
    }

}
