package cn.com.qmth.examcloud.core.oe.student.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.TempConfigService;
import cn.com.qmth.examcloud.support.examing.ExamQuestion;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author chenken
 * @date 2018年9月5日 下午4:57:05
 * @company QMTH
 * @description 考试作答记录controller
 */
@Api(tags = "考试过程中-试题相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/examQuestion")
public class ExamQuestionController extends ControllerSupport {

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private TempConfigService tempConfigService;

    /**
     * 将mongodb中的答过的题和redis中的题目列表合并返回给前端
     * 返回给前端时注意将正确答案和得分置成null
     *
     * @return
     */
    @ApiOperation(value = "考试过程中-获取试题列表")
    @GetMapping("/findExamQuestionList")
    public List<ExamQuestion> findExamQuestionList() {
        User user = getAccessUser();
        return examRecordQuestionsService.findExamQuestionList(user.getUserId());
    }

    /**
     * 获取试题内容
     *
     * @param questionId
     * @return
     */
    @ApiOperation(value = "考试过程中-获取试题内容")
    @GetMapping("/getQuestionContent")
    public String getQuestionContent(@RequestParam String questionId) {
        User user = getAccessUser();
        return examRecordQuestionsService.getQuestionContent(user.getUserId(), questionId);
    }

    /**
     * 考生作答
     *
     * @param examQuestionInfos
     */
    @Deprecated
    @ApiOperation(value = "考试过程中-考生作答（待废弃）")
    @PostMapping("/submitQuestionAnswer")
    public void submitQuestionAnswer(@RequestBody List<ExamStudentQuestionInfo> examQuestionInfos, HttpServletRequest request) {
        User user = getAccessUser();
        if (tempConfigService.checkDisabledForOldApi(user.getRootOrgId())) {
            // 判断学校是否被禁用旧版本API
            throw new StatusException("403X01", "禁止访问！");
        }

        String referer = request.getHeader("REFERER");
        String agent = request.getHeader("USER-AGENT");
        examRecordQuestionsService.submitQuestionAnswer(user.getUserId(), examQuestionInfos, referer, agent, getIp(request));
    }

}
