package cn.com.qmth.examcloud.core.oe.student.api.controller.client;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.FileUtil;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.student.base.OeConstants;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.core.oe.student.bean.client.*;
import cn.com.qmth.examcloud.core.oe.student.service.*;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.examing.ExamQuestion;
import cn.com.qmth.examcloud.support.examing.ExamRecordPaperStruct;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.support.filestorage.UploadResult;
import cn.com.qmth.examcloud.support.handler.richtext2.RichTextConverter;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Api(tags = "（客户端）考试过程相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/client/exam/process")
public class ExamProcessController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ExamProcessController.class);

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordPaperStructService examRecordPaperStructService;

    @Autowired
    private ExamControlService examControlService;

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private FaceProcessService faceProcessService;

    @Autowired
    private ExamFaceLiveVerifyService examFaceLiveVerifyService;

    @Autowired
    private RedisClient redisClient;

    @ApiOperation(value = "开始考试")
    @PostMapping("/startExam")
    public StartExamInfo startExam(@RequestParam Long examStudentId) {
        User user = getAccessUser();
        return examControlService.startExam(examStudentId, user.getUserId(), getIp(getRequest()), null);
    }

    @ApiOperation(value = "开始答题")
    @PostMapping("/startAnswer")
    public StartAnswerInfo startAnswer(@RequestParam @ApiParam(value = "考试记录ID") Long examRecordDataId) {
        User user = getAccessUser();
        return examControlService.startAnswer(examRecordDataId, user.getUserId());
    }

    @ApiOperation(value = "断点续考:检查正在进行中的考试")
    @PostMapping("/checkExamInProgress")
    public ExamProcessResultInfo checkExamInProgress() {
        User user = getAccessUser();
        return examControlService.checkExamInProgress2(user.getUserId(), getIp(getRequest()));
    }

    @ApiOperation(value = "考试心跳")
    @PostMapping("/examHeartbeat")
    public Long examHeartbeat(HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.examHeartbeat(user, getIp(request));
    }

    @ApiOperation(value = "结束考试：交卷")
    @PostMapping("/endExam")
    public void endExam(@RequestParam(required = false) Boolean force, HttpServletRequest request) {
        User user = getAccessUser();
        examControlService.manualEndExam(user.getUserId(), getIp(request), force);
    }

    @ApiOperation(value = "获取考试记录信息")
    @PostMapping("/getEndExamInfo")
    public EndExamInfo getEndExamInfo(@RequestParam Long examRecordDataId) {
        return examControlService.getEndExamInfo(examRecordDataId);
    }

    @ApiOperation(value = "获取考试记录试卷结构")
    @PostMapping("/getExamRecordPaperStruct")
    public ExamRecordPaperStruct getExamRecordPaperStruct(@RequestParam Long examRecordDataId) {
        return examRecordPaperStructService.getExamRecordPaperStruct(examRecordDataId);
    }

    @ApiOperation(value = "考试过程中-获取试题列表")
    @PostMapping("/findExamQuestionList")
    public List<ExamQuestion> findExamQuestionList() {
        User user = getAccessUser();
        List<ExamQuestion> examQuestions = examRecordQuestionsService.findExamQuestionList(user.getUserId());

        JsonMapper jsonMapper = JsonMapper.nonNullMapper();
        for (ExamQuestion examQuestion : examQuestions) {
            if (StringUtils.isBlank(examQuestion.getStudentAnswer())) {
                continue;
            }

            // 将考生作答内容的 HTML结构转换为“富文本”JSON结构
            examQuestion.setStudentAnswer(jsonMapper.toJson(RichTextConverter.parse(examQuestion.getStudentAnswer())));
        }

        return examQuestions;
    }

    @ApiOperation(value = "考试过程中-获取试题内容")
    @PostMapping("/getQuestionContent")
    public String getQuestionContent(@RequestParam String questionId) {
        User user = getAccessUser();
        return examRecordQuestionsService.getQuestionContentForClient(user.getUserId(), questionId);
    }

    @ApiOperation(value = "考试过程中-考生试题作答")
    @PostMapping("/submitQuestionAnswer")
    public void submitQuestionAnswer(@RequestBody List<ExamStudentQuestionInfo> examQuestionInfos, HttpServletRequest request) {
        User user = getAccessUser();
        String referer = request.getHeader("REFERER");
        String agent = request.getHeader("USER-AGENT") + "client-" + Constants.ELECTRON_EXAM_SHELL;
        examRecordQuestionsService.submitQuestionAnswer(user.getUserId(), examQuestionInfos, referer, agent, getIp(request));
    }

    @ApiOperation(value = "获取课程信息")
    @PostMapping("/getCourseInfo/{examRecordDataId}")
    public CourseInfo courseInfo(@PathVariable Long examRecordDataId) {
        return examRecordDataService.getCourseInfo(examRecordDataId);
    }

    @ApiOperation(value = "违纪（非法考生端应用）")
    @PostMapping("/discipline")
    public void discipline(@RequestParam(required = false) String reason) {
        User user = getAccessUser();
        ExamingSession examingSession = examingSessionService.getExamingSession(user.getUserId());
        if (examingSession == null) {
            return;
        }

        if (StringUtils.isNotEmpty(reason)) {
            // 限制字符串最大长度
            reason = StringUtils.substring(reason, 0, 1000);
        }

        String cacheKey = CacheConstants.CACHE_OE_DISCIPLINE_ILLEGAL_CLIENT + examingSession.getExamRecordDataId();
        redisClient.set(cacheKey, reason, OeConstants.TIME_OUT_1_DAY);
    }

    @ApiOperation(value = "文件上传")
    @PostMapping("/upload")
    public UploadResult upload(@RequestParam(required = false) String md5,
                               @RequestPart(value = "file", required = false) MultipartFile file) throws Exception {
        Check.isBlank(md5, "文件MD5不能为空");
        Check.isNull(file, "文件不能为空");

        String realMD5 = DigestUtils.md5Hex(file.getInputStream());
        if (!realMD5.equals(md5)) {
            log.warn("realMD5 = {}, md5 = {}", realMD5, md5);
            throw new StatusException("400403", "文件MD5验证失败");
        }

        User user = getAccessUser();

        final String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
        final String newFileName = FileUtil.generateFileName();
        //路径规则：oe/rootOrgId/yyyyMMdd/userId_fileName.fileSuffix
        final String uploadPath = String.format("oe/%s/%s/%s_%s%s", user.getRootOrgId(), FileUtil.dateDir(), user.getUserId(), newFileName, fileSuffix);

        FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
        env.setRootOrgId(String.valueOf(user.getRootOrgId()));
        env.setRelativePath(uploadPath);
        YunPathInfo result = FileStorageUtil.saveFile(Constants.OE_SITEID, env, file.getBytes(), false);

        return new UploadResult(file.getOriginalFilename(), result.getRelativePath(), result.getUrl());
    }

    @ApiOperation(value = "保存人脸识别比对验证结果")
    @PostMapping("/saveFaceCompareResult")
    public void saveFaceCompareResult(@RequestBody FaceCompareResult req) {
        User user = getAccessUser();
        req.setStudentId(user.getUserId());
        faceProcessService.saveFaceCompareResult(req);
    }

    @ApiOperation(value = "保存人脸抓拍比对验证结果")
    @PostMapping("/saveFaceCaptureResult")
    public void saveFaceCaptureResult(@RequestBody FaceCaptureResult req) {
        User user = getAccessUser();
        req.setStudentId(user.getUserId());
        faceProcessService.saveFaceCaptureResult(req);
    }

    @ApiOperation(value = "开始人脸活体验证")
    @PostMapping("/startFaceLiveVerify")
    public FaceLiveVerifyInfo startFaceLiveVerify(@RequestParam Long examRecordDataId) {
        User user = getAccessUser();
        return examFaceLiveVerifyService.startFaceLiveVerify(examRecordDataId, user.getUserId());
    }

    @ApiOperation(value = "保存人脸活体验证结果")
    @PostMapping("/saveFaceLiveVerifyResult")
    public void saveFaceLiveVerifyResult(@RequestBody FaceLiveVerifyResult req) {
        User user = getAccessUser();
        req.setStudentId(user.getUserId());
        examFaceLiveVerifyService.saveFaceLiveVerifyResult(req);
    }

}
