/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-30 15:54:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.student.base.utils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @author: QMTH
 * @since: 2018/8/30
 */
public class DateUtils {
    public static final String PATTERN = "yyyy-MM-dd";
    public static final String FULL_PATTERN = "yyyy-MM-dd HH:mm:ss";

    /**
     * 日期字符串转日期对象
     *
     * @param dateStr 支持格式：yyyy/MM/dd 或 yyyy-MM-dd
     * @return
     */
    public static Date parse(String dateStr) {
        if (dateStr == null) {
            return null;
        }
        try {
            dateStr = dateStr.replaceAll("/", "-");
            if (dateStr.length() == 10) {
                return new SimpleDateFormat(PATTERN).parse(dateStr);
            }
            return new SimpleDateFormat(FULL_PATTERN).parse(dateStr);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static String format(Date date) {
        if (date == null) {
            return null;
        }
        return new SimpleDateFormat(FULL_PATTERN).format(date);
    }

    /**
     * 计算时长
     */
    public static String diff(long diff) {
        long nd = 1000 * 24 * 60 * 60;//一天的毫秒数
        long nh = 1000 * 60 * 60;//一小时的毫秒数
        long nm = 1000 * 60;//一分钟的毫秒数
        long ns = 1000;//一秒钟的毫秒数
        long day = diff / nd;//计算差多少天
        long hour = diff % nd / nh;//计算差多少小时
        long min = diff % nd % nh / nm;//计算差多少分钟
        long sec = diff % nd % nh % nm / ns;//计算差多少秒//输出结果

        StringBuilder sb = new StringBuilder();
        if (day != 0) {
            sb.append(day + "d ");
        }
        if (hour != 0) {
            sb.append(hour + "h");
        }
        if (min != 0) {
            sb.append(min + "m");
        }
        if (sec != 0) {
            sb.append(sec + "s");
        }
        return sb.toString();
    }
    /**
     * 在指定日期添加固定小时数
     * @param date 日期
     * @param hours 小时数（可为负数）
     * @return
     */
    public static Date addHours(Date date,int hours) {
        return addDateByField(date, Calendar.HOUR_OF_DAY,hours);
    }
    /**
     * 给日期指定区块添加相应的数值
     * @param date 日期
     * @param field 日期区块
     * @param amount 数值(可为负数)
     * @return
     */
    public static Date addDateByField(Date date,int field,int amount) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.add(field,amount);
        return ca.getTime();
    }
}