/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-09-12 09:55:46.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.student.base.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Mysql相关语句处理工具类
 *
 * @author: QMTH
 * @since: 2018/9/5
 */
public class Sentence {
    private static final String YYYY_MM_DD = "yyyy-MM-dd";
    private static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    /**
     * 处理字符串
     */
    public static String dealString(Object value, String defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof CharSequence) {
            return String.valueOf(value);
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to String.");
    }

    public static String dealString(Object value) {
        return dealString(value, null);
    }

    /**
     * 处理Integer
     */
    public static Integer dealInteger(Object value, Integer defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof Number) {
            return ((Number) value).intValue();
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Integer.");
    }

    public static Integer dealInteger(Object value) {
        return dealInteger(value, null);
    }

    /**
     * 处理Long
     */
    public static Long dealLong(Object value, Long defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof Number) {
            return ((Number) value).longValue();
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Long.");
    }

    public static Long dealLong(Object value) {
        return dealLong(value, null);
    }

    /**
     * 处理Double
     */
    public static Double dealDouble(Object value, Double defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof Number) {
            return ((Number) value).doubleValue();
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Double.");
    }

    public static Double dealDouble(Object value) {
        return dealDouble(value, null);
    }

    /**
     * 处理Float
     */
    public static Float dealFloat(Object value, Float defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof Number) {
            return ((Number) value).floatValue();
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Float.");
    }

    public static Float dealFloat(Object value) {
        return dealFloat(value, null);
    }

    /**
     * 处理Boolean
     */
    public static Boolean dealBoolean(Object value, Boolean defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        if (value instanceof Boolean) {
            return (Boolean) value;
        }
        String str = String.valueOf(value);
        final String trueStr = "1";
        if (trueStr.equals(str)) {
            return true;
        }
        final String falseStr = "0";
        if (falseStr.equals(str)) {
            return false;
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Boolean.");
    }

    public static Boolean dealBoolean(Object value) {
        return dealBoolean(value, null);
    }

    /**
     * 处理日期
     */
    public static String dealDate(Object value, Date defaultValue) {
        if (value == null) {
            if (defaultValue != null) {
                return formatDate(defaultValue);
            }
            return null;
        }
        if (value instanceof Date) {
            return formatDate((Date) value);
        }
        //判断是否为日期格式
        String dateStr = String.valueOf(value);
        Date date = parseDate(dateStr);
        if (date != null) {
            return formatDate(date);
        }
        throw new IllegalArgumentException("[" + value + "] Can't convert to Date.");
    }

    public static String dealDate(Object value) {
        return dealDate(value, null);
    }

    public static Date parseDate(String dateStr) {
        if (dateStr == null) {
            return null;
        }
        try {
            final int len = 10;
            if (dateStr.length() == len) {
                return new SimpleDateFormat(YYYY_MM_DD).parse(dateStr);
            }
            return new SimpleDateFormat(YYYY_MM_DD_HH_MM_SS).parse(dateStr);
        } catch (ParseException e) {
            return null;
        }
    }

    public static String formatDate(Date date) {
        return new SimpleDateFormat(YYYY_MM_DD_HH_MM_SS).format(date);
    }

}