package cn.com.qmth.examcloud.core.oe.student.dao;

import cn.com.qmth.examcloud.core.oe.student.dao.entity.FaceBiopsyItemEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;

/**
 * @Description 人脸检测明细仓库
 * @Author lideyin
 * @Date 2019/7/17 18:58
 * @Version 1.0
 */
@Repository
public interface FaceBiopsyItemRepo extends JpaRepository<FaceBiopsyItemEntity, Long>, JpaSpecificationExecutor<FaceBiopsyItemEntity> {

    List<FaceBiopsyItemEntity> findByExamRecordDataIdOrderByIdAsc(Long examRecordDataId);

    /**
     * 根据活检结果id，按明细id正序获取活检明细
     * @param faceBiopsyId 活检最终结果表id
     * @return
     */
    List<FaceBiopsyItemEntity> findByFaceBiopsyIdOrderByIdAsc(Long faceBiopsyId);

    /**
     * 根据活检结果id，按明细id正序获取第一条活检明细(即第一次活检)
     * @param faceBiopsyId 活检最终结果表id
     * @return
     */
    FaceBiopsyItemEntity findFirstByFaceBiopsyIdOrderByIdAsc(Long faceBiopsyId);

    /**
     * 根据活检结果id，按明细id倒序获取第一条活检明细(即最后一次活检)
     * @param faceBiopsyId 活检最终结果表id
     * @return
     */
    FaceBiopsyItemEntity findFirstByFaceBiopsyIdOrderByIdDesc(Long faceBiopsyId);

    /**
     * 根据考试记录id，按明细id正序获取第一条活检明细(即第一次活检)
     * @param examRecordDataId 考试记录id
     * @return
     */
    FaceBiopsyItemEntity findFirstByExamRecordDataIdOrderByIdAsc(Long examRecordDataId);

    /**
     * 根据考试记录id和人检检测完成状态获取数据
     *
     * @param examRecordDataId 考试记录id
     * @param completed        是否检测完成
     * @return
     */
    List<FaceBiopsyItemEntity> findByExamRecordDataIdAndCompleted(Long examRecordDataId, Boolean completed);

    @Transactional
    @Modifying
    @Query(nativeQuery = true,
            value = "update ec_oes_exam_face_biopsy_item set result=?2,error_msg=?3,completed=?4 where id=?1")
    void updateFaceBiopsyItemResult(Long id, boolean result, String errorMsg, boolean completed);
}
