package cn.com.qmth.examcloud.core.oe.student.report;

import cn.com.qmth.examcloud.core.oe.student.service.ExamProcessRecordService;
import cn.com.qmth.examcloud.reports.commons.enums.Tag;
import cn.com.qmth.examcloud.reports.commons.enums.Topic;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Properties;

public class RocketMqConsumerListener {
    private static final Logger LOG = LoggerFactory.getLogger(RocketMqConsumerListener.class);

    private static ExamProcessRecordService examProcessRecordService = SpringContextHolder.getBean(ExamProcessRecordService.class);
    private static Properties properties = new Properties();

    static {
        properties.put(PropertyKeyConst.AccessKey, PropertyHolder.getString("$rocketmq-accesskey"));
        // AccessKeySecret 阿里云身份验证，在阿里云服务器管理控制台创建。
        properties.put(PropertyKeyConst.SecretKey, PropertyHolder.getString("$rocketmq-secretkey"));
        // 设置发送超时时间，单位毫秒。
        properties.setProperty(PropertyKeyConst.SendMsgTimeoutMillis, "3000");
        // 设置 TCP 接入域名，进入控制台的实例详情页面的 TCP 协议客户端接入点区域查看。
        properties.put(PropertyKeyConst.NAMESRV_ADDR, PropertyHolder.getString("$rocketmq-namesrv-addr"));
        // 顺序消息消费失败进行重试前的等待时间，单位（毫秒），取值范围: 10 毫秒 ~ 30,000 毫秒
        properties.put(PropertyKeyConst.SuspendTimeMillis, "100");
        // 消息消费失败时的最大重试次数
        properties.put(PropertyKeyConst.MaxReconsumeTimes, "10");
    }

    public static void start() {
        onlineExamStudent();
    }

    private static void onlineExamStudent() {
        properties.put(PropertyKeyConst.GROUP_ID, Tag.EXAM_PROCESS_RECORD.getGroup());
        Consumer consumer = ONSFactory.createConsumer(properties);
        consumer.subscribe(Topic.REPORT_TOPIC.getCode(), Tag.EXAM_PROCESS_RECORD.getCode(), new MessageListener() {
            @Override
            public Action consume(Message message, ConsumeContext context) {
                try {
                    String msg = new String(message.getBody(), "utf-8");
                    onMessageExamStudent(msg);
                    return Action.CommitMessage;
                } catch (Exception e) {
                    LOG.error("consumer failed MsgID:" + message.getMsgID(), e);
                    return Action.ReconsumeLater;
                }
            }

        });

        consumer.start();
    }

    private static void onMessageExamStudent(String message) {
        ExamProcessRecordReport r = JSON.parseObject(message, ExamProcessRecordReport.class);
        examProcessRecordService.saveExamProcessRecord(r.getExamRecordDataId(),
                r.getExamProcess().getDesc(), r.getRecordTime(), r.getRemoteHost());
    }
}
