package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import cn.com.qmth.examcloud.commons.util.UrlUtil;
import cn.com.qmth.examcloud.core.oe.student.bean.task.SaveExamCaptureQueueInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.enums.ExamCaptureQueueStatus;
import cn.com.qmth.examcloud.core.oe.student.service.ExamCaptureQueueService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author chenken
 * @date 2018/8/17 14:48
 * @company QMTH
 * @description 考试抓拍处理队列服务实现
 */
@Service("examCaptureQueueService")
public class ExamCaptureQueueServiceImpl implements ExamCaptureQueueService {

    private static final Logger log = LoggerFactory.getLogger(ExamCaptureQueueServiceImpl.class);

    @Autowired
    private ExamCaptureQueueRepo examCaptureQueueRepo;

    @Override
    public boolean saveExamCaptureQueueEntityByFailed(Long captureQueueId, String errorMsg,
                                                      ExamCaptureQueueStatus examCaptureQueueStatus) {
        try {
            examCaptureQueueRepo.saveExamCaptureQueueEntityByFailed(captureQueueId, errorMsg, examCaptureQueueStatus.toString(),
                    "000000", new Date());
            return true;
        } catch (Exception e) {
            log.error("[UPDATE_FAILED_CAPTURE_QUEUE] 保存照片队列处理失败数据时出现异常", e);
            return false;
        }
    }

    @Override
    public String saveExamCaptureQueue(SaveExamCaptureQueueInfo saveExamCaptureQueueInfo, Long studentId) {
        //查询学生底照faceToken
        StudentCacheBean studentCache = CacheHelper.getStudent(studentId);
        String baseFaceToken = studentCache.getFaceToken();
        if (StringUtils.isBlank(baseFaceToken)) {
            throw new StatusException("300002", "学生底照的faceToken为空");
        }

        String fileUrl = UrlUtil.decode(saveExamCaptureQueueInfo.getFileUrl());
        if (!StringUtil.isAscString(fileUrl)) {
            log.error("The fileUrl is invalid:" + saveExamCaptureQueueInfo.getFileUrl());
            throw new StatusException("300001", "文件路径格式不正确");
        }
        String fileName = fileUrl.substring(fileUrl.lastIndexOf("/") + 1);

        ExamCaptureQueueEntity examCaptureQueue = new ExamCaptureQueueEntity();
        examCaptureQueue.setStudentId(studentId);
        examCaptureQueue.setFileUrl(fileUrl);
        examCaptureQueue.setFileName(fileName);
        examCaptureQueue.setBaseFaceToken(baseFaceToken);
        examCaptureQueue.setExamRecordDataId(saveExamCaptureQueueInfo.getExamRecordDataId());
        examCaptureQueue.setHasVirtualCamera(saveExamCaptureQueueInfo.getHasVirtualCamera());

        if (StringUtils.length(saveExamCaptureQueueInfo.getCameraInfos()) >= Constants.VM_CAMERA_SIZE_LIMIT) {
            examCaptureQueue.setCameraInfos(Constants.VM_CAMERA_WARN);
            log.warn("虚拟摄像头信息超长！ " + saveExamCaptureQueueInfo.getExamRecordDataId());
        } else {
            examCaptureQueue.setCameraInfos(saveExamCaptureQueueInfo.getCameraInfos());
        }

        examCaptureQueue.setExtMsg(saveExamCaptureQueueInfo.getExtMsg());
        examCaptureQueue.setStatus(ExamCaptureQueueStatus.PENDING);
        examCaptureQueue.setErrorNum(0);
        examCaptureQueue.setCreationTime(new Date());
        examCaptureQueueRepo.save(examCaptureQueue);
        return fileName;
    }


}
