package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamProcessRecordInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamProcessRecordRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamProcessRecordEntity;
import cn.com.qmth.examcloud.core.oe.student.service.ExamProcessRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description 考试过程记录
 * @Author lideyin
 * @Date 2020/8/20 17:49
 * @Version 1.0
 */
@Service("examProcessRecordService")
public class ExamProcessRecordServiceImpl implements ExamProcessRecordService {

    @Autowired
    private ExamProcessRecordRepo examProcessRecordRepo;

    /**
     * 保存考试记录
     *
     * @param examRecordDataId
     * @param processName
     * @param recordTime
     * @param sourceIp
     */
    @Override
    public void saveExamProcessRecord(Long examRecordDataId, String processName, Date recordTime, String sourceIp) {
        ExamProcessRecordEntity entity = new ExamProcessRecordEntity();
        entity.setExamRecordDataId(examRecordDataId);
        entity.setProcessName(processName);
        entity.setRecordTime(recordTime);
        entity.setSourceIp(sourceIp);

        examProcessRecordRepo.save(entity);
    }

    /**
     * 获取考试过程记录
     *
     * @param examRecordDataId
     * @return
     */
    @Override
    public List<ExamProcessRecordInfo> getExamProcessRecords(Long examRecordDataId) {
        List<ExamProcessRecordEntity> recordList =
                examProcessRecordRepo.findByExamRecordDataIdOrderByRecordTime(examRecordDataId);
        List<ExamProcessRecordInfo> resultList = new ArrayList<>();
        if (null != recordList && !recordList.isEmpty()) {
            for (ExamProcessRecordEntity record : recordList) {
                ExamProcessRecordInfo info = new ExamProcessRecordInfo();
                info.setId(record.getId());
                info.setExamRecordDataId(record.getExamRecordDataId());
                info.setProcessName(record.getProcessName());
                info.setRecordTime(DateUtil.format(record.getRecordTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
                info.setSourceIp(record.getSourceIp());

                resultList.add(info);
            }
        }

        return resultList;
    }

}
