package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordPaperStructReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordPaperStructResp;
import cn.com.qmth.examcloud.core.oe.student.base.OeConstants;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordPaperStructService;
import cn.com.qmth.examcloud.support.examing.ExamRecordPaperStruct;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("examRecordPaperStructService")
public class ExamRecordPaperStructServiceImpl implements ExamRecordPaperStructService {

    @Autowired
    private RedisClient redisClient;


    @Override
    public void saveExamRecordPaperStruct(Long examRecordDataId, ExamRecordPaperStruct paper) {
        String key = RedisKeyHelper.getBuilder().studentPaperKey(examRecordDataId);
        redisClient.set(key, paper, OeConstants.TIME_OUT_7_DAY);
    }

    @Override
    public ExamRecordPaperStruct getExamRecordPaperStruct(Long examRecordDataId) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        String key = RedisKeyHelper.getBuilder().studentPaperKey(examRecordDataId);
        return redisClient.get(key, ExamRecordPaperStruct.class);
    }

    @Override
    public void deleteExamRecordPaperStruct(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().studentPaperKey(examRecordDataId);
        redisClient.delete(key);
    }

    @Override
    public GetExamRecordPaperStructResp getExamRecordPaperStruct(GetExamRecordPaperStructReq req) {
        ExamRecordPaperStruct examRecordPaperStruct = this.getExamRecordPaperStruct(req.getExamRecordDataId());

        if (null == examRecordPaperStruct) {
            throw new StatusException("200001", "找不到该考试的试卷结构");
        }

        GetExamRecordPaperStructResp resp = new GetExamRecordPaperStructResp();
        resp.setId(examRecordPaperStruct.getId());
        resp.setDefaultPaper(examRecordPaperStruct.getDefaultPaper());

        return resp;
    }

}
