package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.student.api.bean.StuExamQuestionBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordQuestionsReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordQuestionsResp;
import cn.com.qmth.examcloud.core.oe.student.base.OeConstants;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionAnswerInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordQuestionTempRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamQuestionTempEntity;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamingSessionService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestion;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;
import cn.com.qmth.examcloud.starters.crypto.CryptoProperties;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoGroup;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoHelper;
import cn.com.qmth.examcloud.starters.crypto.common.FieldPair;
import cn.com.qmth.examcloud.starters.crypto.service.CryptoFactory;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.QuestionCacheBean;
import cn.com.qmth.examcloud.support.examing.*;
import cn.com.qmth.examcloud.support.handler.QuestionBodyHandler;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordQuestionsServiceImpl.class);

    /**
     * 考生作答内容长度界限，超过此长度的答案内容存入mongo
     */
    private final static int ANWSER_LENGT = 20000;

    // 本地题干缓存，最大200道题，默认缓存120秒
    private Cache<String, String> questionContentCache = CacheBuilder.newBuilder().maximumSize(200)
            .expireAfterWrite(120, TimeUnit.SECONDS).build();

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordQuestionTempRepo examRecordQuestionTempRepo;

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamControlService examControlService;

    @Autowired
    private CryptoFactory cryptoFactory;

    @Autowired
    private CryptoProperties cryptoProperties;

    @Override
    public void saveExamQuestion(Long examRecordDataId, Integer order, ExamQuestion question) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.set(key, question, OeConstants.TIME_OUT_7_DAY);
    }

    @Override
    public ExamQuestion getExamQuestion(Long examRecordDataId, Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        return redisClient.get(key, ExamQuestion.class);
    }

    @Override
    public void deleteExamQuestion(Long examRecordDataId, Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.delete(key);
    }

    @Override
    public ExamRecordQuestions createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestions examRecordQuestions = new ExamRecordQuestions();

        List<ExamQuestion> examQuestionEntityList = new ArrayList<ExamQuestion>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup
                    .getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper
                        .getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestion examQuestionEntity = new ExamQuestion();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setIsInMongo(false);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestions.setExamQuestions(examQuestionEntityList);
        examRecordQuestions.setExamRecordDataId(examRecordDataId);
        examRecordQuestions.setCreationTime(new Date());
        for (ExamQuestion eq : examRecordQuestions.getExamQuestions()) {
            saveExamQuestion(eq.getExamRecordDataId(), eq.getOrder(), eq);
        }
        return examRecordQuestions;
    }

    @Override
    public List<ExamQuestion> findExamQuestionList(Long studentId) {
        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("1001", "考试会话已过期，请重新开考");
        }

        String examingHeartbeatKey = RedisKeyHelper.getBuilder().examingHeartbeatKey(examSessionInfo.getExamRecordDataId());
        ExamingHeartbeat examingHeartbeat = redisClient.get(examingHeartbeatKey, ExamingHeartbeat.class);
        if (null != examingHeartbeat
                && examingHeartbeat.getCost() >= examSessionInfo.getExamDuration()) {
            throw new StatusException("1001", "考试会话已过期，请重新开考");
        }

        ExamRecordQuestions recordQuestions = this.getExamRecordQuestions(examSessionInfo.getExamRecordDataId());
        List<ExamQuestion> examQuestionList = recordQuestions.getExamQuestions();
        for (ExamQuestion examQuestion : examQuestionList) {
            examQuestion.setCorrectAnswer(null);
            examQuestion.setStudentScore(null);

            if (QuestionType.SINGLE_CHOICE == examQuestion.getQuestionType() ||
                    QuestionType.MULTIPLE_CHOICE == examQuestion.getQuestionType()) {
                // 选择题 - 转换为新答案格式（兼容旧格式）
                // examQuestion.setStudentAnswer(QuestionOptionHelper.parseNumbers(examQuestion.getStudentAnswer()));
            }
        }
        return examQuestionList;
    }

    @Override
    public ExamRecordQuestions getExamRecordQuestions(Long examRecordDataId) {
        ExamRecordData examRecordDataCache = examRecordDataService.getExamRecordDataCache(examRecordDataId);
        if (examRecordDataCache == null) {
            throw new StatusException("1001", "考试记录缓存信息不存在！" + examRecordDataId);
        }

        ExamRecordQuestions result = new ExamRecordQuestions();
        result.setExamRecordDataId(examRecordDataId);
        List<ExamQuestion> examQuestions = new ArrayList<>();
        result.setExamQuestions(examQuestions);

        // 试卷题目数量
        Integer quesCount = examRecordDataCache.getQuestionCount();
        for (int i = 1; i <= quesCount; i++) {
            ExamQuestion questionCache = this.getExamQuestion(examRecordDataId, i);
            if (questionCache == null) {
                log.warn("ExamQuestion cache not exist, examRecordDataId:{}, order:{}", examRecordDataId, i);
                throw new StatusException("1002", "试题作答记录不存在！");
            }

            if (questionCache.getIsInMongo()) {
                // 若记录存在mongo中，需要从mongo中获取（考生作答内容过长时）
                Optional<ExamQuestionTempEntity> op = examRecordQuestionTempRepo.findById(questionCache.getExamQuestionTempId());
                if (!op.isPresent()) {
                    throw new StatusException("1002", "试题作答信息不存在！" + questionCache.getExamQuestionTempId());
                }

                this.copyExamQuestionInfo(questionCache, op.get());
                log.warn("ExamQuestion data in mongodb, examQuestionTempId:{}", questionCache.getExamQuestionTempId());
            }

            examQuestions.add(questionCache);
        }

        return result;
    }

    @Override
    public String getQuestionContent(Long studentId, String questionId) {
        Check.isBlank(questionId, "questionId不能为空");

        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("2001", "考试已结束，不允许获取试题内容");
        }

        // 本地缓存key规则：examId_courseCode_paperType_questionId
        String cacheKey = examSessionInfo.getExamId() + "_" + examSessionInfo.getCourseCode() + "_"
                + examSessionInfo.getPaperType() + "_" + questionId;
        String contentJson = questionContentCache.getIfPresent(cacheKey);

        // 如果本地缓存中存在，则从本地缓存中获取
        if (StringUtils.isNotEmpty(contentJson)) {
            return contentJson;
        } else {// 如果本地缓存不存在，则从redis中获取, 并在本地缓存存在存储一份
            QuestionCacheBean getQuestionResp = CacheHelper.getQuestion(examSessionInfo.getExamId(),
                    examSessionInfo.getCourseCode(), examSessionInfo.getPaperType(), questionId);

            DefaultQuestion defaultQuestion = getQuestionResp.getDefaultQuestion();
            DefaultQuestionStructure questionStructure = defaultQuestion.getMasterVersion();
            List<DefaultQuestionUnit> questionUnits = questionStructure.getQuestionUnitList();

            // 在线考试,清除答案
            if (ExamType.ONLINE.name().equals(examSessionInfo.getExamType())
                    || ExamType.ONLINE_HOMEWORK.name().equals(examSessionInfo.getExamType())) {
                for (DefaultQuestionUnit questionUnit : questionUnits) {
                    questionUnit.setRightAnswer(null);
                }
            }
            String resultJson = JsonUtil.toJson(questionStructure);
            questionContentCache.put(cacheKey, resultJson);
            return resultJson;
        }
    }

    @Override
    public String getQuestionContentForClient(Long studentId, String questionId) {
        Check.isBlank(questionId, "questionId不能为空");
        Check.isNull(studentId, "studentId不能为空");

        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("2001", "考试已结束，不允许获取试题内容");
        }

        // 本地缓存key规则：examId_courseCode_paperType_questionId
        String cacheKey = "CLIENT_" + examSessionInfo.getExamId() + "_" + examSessionInfo.getCourseCode() + "_"
                + examSessionInfo.getPaperType() + "_" + questionId;
        String contentJson = questionContentCache.getIfPresent(cacheKey);

        // 如果本地缓存中存在，则从本地缓存中获取
        if (StringUtils.isNotEmpty(contentJson)) {
            return contentJson;
        } else {// 如果本地缓存不存在，则从redis中获取, 并在本地缓存存在存储一份
            QuestionCacheBean getQuestionResp = CacheHelper.getQuestion(examSessionInfo.getExamId(),
                    examSessionInfo.getCourseCode(), examSessionInfo.getPaperType(), questionId);

            DefaultQuestion defaultQuestion = getQuestionResp.getDefaultQuestion();
            DefaultQuestionStructure questionStructure = defaultQuestion.getMasterVersion();
            List<DefaultQuestionUnit> questionUnits = questionStructure.getQuestionUnitList();

            // 将题干、选项等 HTML结构转换为“富文本”JSON结构
            QuestionBodyHandler.convertRichText(defaultQuestion);

            // 在线考试,清除答案
            if (ExamType.ONLINE.name().equals(examSessionInfo.getExamType())
                    || ExamType.ONLINE_HOMEWORK.name().equals(examSessionInfo.getExamType())) {
                for (DefaultQuestionUnit questionUnit : questionUnits) {
                    questionUnit.setRightAnswer(null);
                }
            }
            String resultJson = JsonUtil.toJson(questionStructure);
            questionContentCache.put(cacheKey, resultJson);
            return resultJson;
        }
    }

    @Override
    public void submitQuestionAnswer(Long studentId, List<ExamStudentQuestionInfo> examQuestionInfos, String referer, String agent, String ip) {
        if (CollectionUtils.isEmpty(examQuestionInfos)) {
            log.warn("submitQuestionAnswer is empty. studentId = {} ", studentId);
            return;
        }

        for (ExamStudentQuestionInfo examStudentQuestionInfo : examQuestionInfos) {
            if (examStudentQuestionInfo.getOrder() == null) {
                throw new StatusException("2001", "题目序号不能为空");
            }
        }

        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("3001", "考试会话已过期");
        }
        long examRecordDataId = examSessionInfo.getExamRecordDataId();

        if (StringUtils.isEmpty(referer) || StringUtils.isEmpty(agent) || !agent.contains(Constants.ELECTRON_EXAM_SHELL)) {
            String cacheKey = CacheConstants.CACHE_OE_DISCIPLINE_ILLEGAL_DATA + examSessionInfo.getExamRecordDataId();
            redisClient.set(cacheKey, true, OeConstants.TIME_OUT_1_DAY);
        }

        for (ExamStudentQuestionInfo examQuestionInfo : examQuestionInfos) {
            ExamQuestion eq = getExamQuestion(examRecordDataId, examQuestionInfo.getOrder());
            if (eq == null) {
                log.warn("ExamQuestion cache not exist, examRecordDataId:{}, order:{}", examRecordDataId, examQuestionInfo.getOrder());
                throw new StatusException("2002", "试题作答记录不存在！");
            }

            eq.setStudentAnswer(examQuestionInfo.getStudentAnswer());
            eq.setIsSign(examQuestionInfo.getIsSign());
            eq.setIsAnswer(StringUtils.isNotBlank(eq.getStudentAnswer()));
            eq.setAudioPlayTimes(examQuestionInfo.getAudioPlayTimes());

            //过长存入mongo
            if (eq.getIsAnswer() && eq.getStudentAnswer().length() > ANWSER_LENGT) {
                eq.setIsInMongo(true);
                ExamQuestionTempEntity temp = ofExamQuestionTempEntity(eq);
                examRecordQuestionTempRepo.save(temp);
                eq.setExamQuestionTempId(temp.getId());
                eq.setStudentAnswer(null);
            } else {
                eq.setIsInMongo(false);
            }

            saveExamQuestion(examRecordDataId, examQuestionInfo.getOrder(), eq);
        }

        // 设置并保存上次活动时间
        examControlService.setAndSaveActiveTime(examRecordDataId, ip, "submitQuestionAnswer");
    }

    @Override
    public void submitQuestionAnswerWithCrypto(ExamStudentQuestionAnswerInfo data, User user, String referer, String agent, String timestampStr, String ip) {
        long timestamp;
        try {
            timestamp = CryptoHelper.parseTimestamp(timestampStr);
        } catch (Exception e) {
            log.error("[header] timestamp is wrong... {}", timestampStr);
            throw new StatusException("400X01", CryptoConstant.REQUEST_PARAM_ERROR);
        }

        String key = CryptoHelper.buildKey(
                new FieldPair("key", user.getKey()),
                new FieldPair("token", user.getToken()),
                new FieldPair("timestamp", String.valueOf(timestamp))
        );

        CryptoGroup cryptoGroup = new CryptoGroup(user.getSalt()).matchKeys(key);

        // 按加密方案组合依次解密（倒序）
        String signResult = cryptoFactory.decrypt(cryptoGroup, data.getSign(), true);
        String answerMd5 = DigestUtils.md5Hex(data.getAnswers());
        if (!signResult.equals(answerMd5)) {
            log.warn("[param] answers sign invalid... {} not equal {}", answerMd5, signResult);
            throw new StatusException("400X01", CryptoConstant.REQUEST_PARAM_ERROR);
        }

        List<ExamStudentQuestionInfo> answers = new JsonMapper().toList(data.getAnswers(), ExamStudentQuestionInfo.class);

        this.submitQuestionAnswer(user.getUserId(), answers, referer, agent, ip);
    }

    @Override
    public GetExamRecordQuestionsResp getExamRecordQuestions(GetExamRecordQuestionsReq req) {
        ExamRecordQuestions examRecordQuestions = this.getExamRecordQuestions(req.getExamRecordDataId());
        if (null == examRecordQuestions) {
            throw new StatusException("200002", "当前考试的作答记录不存在！");
        }

        GetExamRecordQuestionsResp resp = new GetExamRecordQuestionsResp();
        resp.setExamRecordDataId(examRecordQuestions.getExamRecordDataId());
        resp.setCreationTime(examRecordQuestions.getCreationTime());

        List<StuExamQuestionBean> beanList = new ArrayList<>();
        List<ExamQuestion> examQuestions = examRecordQuestions.getExamQuestions();

        for (ExamQuestion examQuestion : examQuestions) {
            StuExamQuestionBean bean = new StuExamQuestionBean();
            bean.setInMongo(examQuestion.getIsInMongo());
            bean.setExamQuestionTempId(examQuestion.getExamQuestionTempId());
            bean.setExamRecordDataId(examQuestion.getExamRecordDataId());
            bean.setMainNumber(examQuestion.getMainNumber());
            bean.setQuestionId(examQuestion.getQuestionId());
            bean.setOrder(examQuestion.getOrder());
            bean.setQuestionScore(examQuestion.getQuestionScore());
            bean.setQuestionType(examQuestion.getQuestionType());
            bean.setCorrectAnswer(examQuestion.getCorrectAnswer());

            if (StringUtils.isNotBlank(examQuestion.getStudentAnswer())) {
                bean.setStudentAnswer(examQuestion.getStudentAnswer());
            } else {
                // 作答内容为空时，若记录存在mongo中，需要从mongo中重新获取（考生作答内容过长时）
                if (null != examQuestion.getIsInMongo() && examQuestion.getIsInMongo()) {
                    Optional<ExamQuestionTempEntity> op = examRecordQuestionTempRepo.findById(examQuestion.getExamQuestionTempId());
                    if (!op.isPresent()) {
                        throw new StatusException("200003", "试题作答结果不存在！" + examQuestion.getExamQuestionTempId());
                    }

                    bean.setStudentAnswer(op.get().getStudentAnswer());
                    log.warn("ExamQuestion studentAnswer in mongodb, examQuestionTempId:{}", examQuestion.getExamQuestionTempId());
                }
            }

            bean.setStudentScore(examQuestion.getStudentScore());
            bean.setAnswer(examQuestion.getIsAnswer());
            bean.setSign(examQuestion.getIsSign());
            bean.setOptionPermutation(examQuestion.getOptionPermutation());
            bean.setAudioPlayTimes(examQuestion.getAudioPlayTimes());
            bean.setAnswerType(examQuestion.getAnswerType());

            beanList.add(bean);
        }

        resp.setExamQuestions(beanList);
        return resp;
    }

    private ExamQuestionTempEntity ofExamQuestionTempEntity(ExamQuestion eq) {
        ExamQuestionTempEntity tem = new ExamQuestionTempEntity();
        tem.setExamRecordDataId(eq.getExamRecordDataId());
        tem.setMainNumber(eq.getMainNumber());
        tem.setQuestionId(eq.getQuestionId());
        tem.setOrder(eq.getOrder());
        tem.setQuestionScore(eq.getQuestionScore());
        tem.setQuestionType(eq.getQuestionType());
        tem.setCorrectAnswer(eq.getCorrectAnswer());
        tem.setStudentAnswer(eq.getStudentAnswer());
        tem.setStudentScore(eq.getStudentScore());
        tem.setIsAnswer(eq.getIsAnswer());
        tem.setIsSign(eq.getIsSign());
        tem.setOptionPermutation(eq.getOptionPermutation());
        tem.setAudioPlayTimes(eq.getAudioPlayTimes());
        tem.setAnswerType(eq.getAnswerType());
        return tem;
    }

    private void copyExamQuestionInfo(ExamQuestion question, ExamQuestionTempEntity temp) {
        question.setExamRecordDataId(temp.getExamRecordDataId());
        question.setMainNumber(temp.getMainNumber());
        question.setQuestionId(temp.getQuestionId());
        question.setOrder(temp.getOrder());
        question.setQuestionScore(temp.getQuestionScore());
        question.setQuestionType(temp.getQuestionType());
        question.setCorrectAnswer(temp.getCorrectAnswer());
        question.setStudentAnswer(temp.getStudentAnswer());
        question.setStudentScore(temp.getStudentScore());
        question.setIsAnswer(temp.getIsAnswer());
        question.setIsSign(temp.getIsSign());
        question.setOptionPermutation(temp.getOptionPermutation());
        question.setAudioPlayTimes(temp.getAudioPlayTimes());
        question.setAnswerType(temp.getAnswerType());
    }

}
