package cn.com.qmth.examcloud.core.oe.student.test;

import cn.com.qmth.examcloud.commons.helpers.poi.ExcelReader;
import cn.com.qmth.examcloud.commons.util.FileUtil;
import cn.com.qmth.examcloud.starters.face.verify.FaceVerifyProperties;
import cn.com.qmth.examcloud.starters.face.verify.common.JsonHelper;
import cn.com.qmth.examcloud.starters.face.verify.model.FaceResult;
import cn.com.qmth.examcloud.starters.face.verify.model.param.ImageFileParm;
import cn.com.qmth.examcloud.starters.face.verify.service.impl.FaceVerifyServiceImpl;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.junit.Test;

import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

public class FaceVerifyTest {

    private ExecutorService service = Executors.newCachedThreadPool();

    private AtomicInteger totalNum = new AtomicInteger();

    private AtomicInteger passNum = new AtomicInteger();

    private static Map<String, Double> scores = new ConcurrentHashMap<>();

    private static final FaceVerifyServiceImpl faceVerifyService;

    private static final String dir = "D:\\home\\captures";

    private static final String fileName = "人脸比对-不通过";

    static {
        System.setProperty("log.commonLevel", "INFO");

        FaceVerifyProperties properties = new FaceVerifyProperties();
        properties.setBaiduLocalEnabled(true);
        properties.setBaiduLocalAppId("test");
        properties.setBaiduLocalUrlPrefix("http://192.168.20.253");
        // properties.setBaiduLocalUrlPrefix("http://192.168.0.118");

        faceVerifyService = new FaceVerifyServiceImpl();
        faceVerifyService.setProperties(properties);
    }

    // @Test
    public void demo() throws Exception {
        String excelFile = dir + "/" + fileName + ".xlsx";
        List<String[]> lines = ExcelReader.readSheetBySax(excelFile, 1, 8);

        int batchNum = 50;
        int totalLines = lines.size() - 1;
        List<String[]> batchList = new ArrayList<>();
        long startTime = System.currentTimeMillis();

        for (int n = 1; n < lines.size(); n++) {
            if (n > 100) {
                // break;
            }

            totalNum.incrementAndGet();
            String[] line = lines.get(n);
            batchList.add(line);

            if (n % batchNum == 0) {
                concurrentRun(batchList);
                batchList.clear();

                long cost = (System.currentTimeMillis() - startTime) / 1000L;
                System.out.printf("lineIndex:%s  进度：%s%%  passNum:%s  耗时：%s秒 %n", n, n * 100f / totalLines, passNum.get(), cost);
            }
        }

        if (!batchList.isEmpty()) {
            concurrentRun(batchList);
        }

        long cost = (System.currentTimeMillis() - startTime) / 1000L;
        float passRate = passNum.get() * 100f / totalNum.get();
        System.out.printf("totalLines:%s  passNum:%s  通过率：%s%%  不通过率：%s%%  耗时：%s秒 %n%n", totalNum.get(), passNum.get(), passRate, 100 - passRate, cost);

        // 重新计算不同阈值下的通过率
        Map<Double, Integer> statisticMaps = new TreeMap<>();
        for (int n = 65; n <= 80; n++) {
            statisticMaps.put((double) n, 0);
        }
        for (Map.Entry<String, Double> entry : scores.entrySet()) {
            for (Map.Entry<Double, Integer> map : statisticMaps.entrySet()) {
                if (entry.getValue() >= map.getKey()) {
                    statisticMaps.put(map.getKey(), map.getValue() + 1);
                }
            }
        }
        for (Map.Entry<Double, Integer> map : statisticMaps.entrySet()) {
            double newPassRate = round(map.getValue() * 100d / totalNum.get(), 2);
            System.out.printf("百度阈值：%s  通过率：%s%%  不通过率：%s%% %n", map.getKey(), newPassRate, round(100d - newPassRate, 2));
        }

        service.shutdown();
    }

    private void concurrentRun(List<String[]> batchList) {
        int threadNum = batchList.size();
        final CountDownLatch commander = new CountDownLatch(1);
        final CountDownLatch worker = new CountDownLatch(threadNum);

        for (int n = 0; n < threadNum; n++) {
            String[] line = batchList.get(n);
            final String studentId = line[0];
            final String capturePhotoUrl = fixPhotoUrl(line[6], false);
            final String photoUrl = fixPhotoUrl(line[7], true);
            // System.out.println(studentId + " | " + photoUrl + " | " + capturePhotoUrl);

            Runnable runnable = () -> {
                try {
                    // System.out.println(Thread.currentThread().getName());

                    String photoPath = dir + "/" + fileName + "/" + studentId + "/" + FileUtil.getFileName(photoUrl);
                    File photoFile = new File(photoPath);
                    if (!photoFile.exists()) {
                        FileUtil.saveImageToFile(photoUrl, photoPath);
                    }

                    String capturePhotoPath = dir + "/" + fileName + "/" + studentId + "/" + FileUtil.getFileName(capturePhotoUrl);
                    File capturePhotoFile = new File(capturePhotoPath);
                    if (!capturePhotoFile.exists()) {
                        FileUtil.saveImageToFile(capturePhotoUrl, capturePhotoPath);
                    }

                    String resultPath = dir + "/" + fileName + "/results/" + DigestUtils.md5Hex(studentId + capturePhotoUrl) + ".log";
                    File resultFile = new File(resultPath);
                    if (resultFile.exists()) {
                        String x = FileUtil.readFile(resultFile).get(0);
                        if (x.contains("true")) {
                            passNum.incrementAndGet();
                        } else {
                            // System.out.println(x + " | " + photoPath + " | " + capturePhotoPath);
                        }
                        scores.put(photoUrl + "|" + capturePhotoUrl, Double.valueOf(x.split("\\|")[1]));
                    } else {
                        FaceResult result = faceVerifyService.faceCompareByBaidu(new ImageFileParm(photoPath), new ImageFileParm(capturePhotoPath));
                        if (result.isApiNeedRetry()) {
                            System.out.println(new JsonHelper().toJson(result));
                            return;
                        }

                        if (result.isFacePass()) {
                            passNum.incrementAndGet();
                        }

                        scores.put(photoUrl + "|" + capturePhotoUrl, result.getFaceScore());

                        String x = result.isFacePass() + "|" + result.getFaceScore();
                        FileUtil.saveToFile(x.getBytes(StandardCharsets.UTF_8), resultFile);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    worker.countDown();
                }
            };

            service.execute(runnable);
        }

        try {
            commander.countDown();
            worker.await();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String fixPhotoUrl(String url, boolean isBasePhoto) {
        if (StringUtils.isEmpty(url)) {
            return "";
        }

        if (url.toLowerCase().startsWith("http")) {
            return url;
        }

        final String prefix = "https://ecs-static.qmth.com.cn/";
        if (url.startsWith("aliyun-1://")) {
            return url.replace("aliyun-1://", prefix);
        } else if (url.startsWith("aliyun-2://")) {
            return url.replace("aliyun-2://", prefix);
        } else if (url.startsWith("upyun-1://")) {
            return url.replace("upyun-1://", prefix);
        } else {
            if (isBasePhoto && !url.contains("student_base_photo")) {
                url = "student_base_photo/" + url;
            }
        }

        return prefix + url;
    }

    private double round(double value, int n) {
        if (n < 0) n = 0;
        BigDecimal _value = new BigDecimal(Double.toString(value));
        BigDecimal one = new BigDecimal("1");
        return _value.divide(one, n, RoundingMode.HALF_UP).doubleValue();
    }

}
